@@ -1,3 +1,3 @@
 # flake8: noqa
 from compressor.filters.base import (FilterBase, CallbackOutputFilter,
-                                     CompilerFilter, FilterError)
+                                     CompilerFilter, CachedCompilerFilter, FilterError)
