@@ -1,6 +1,7 @@
 from __future__ import with_statement, unicode_literals
 import os
 import re
+import sys
 from tempfile import mkdtemp
 from shutil import rmtree, copytree
 
@@ -25,8 +26,8 @@ def make_soup(markup):
 
 
 def css_tag(href, **kwargs):
-    rendered_attrs = ''.join(['%s="%s" ' % (k, v) for k, v in kwargs.items()])
-    template = '<link rel="stylesheet" href="%s" type="text/css" %s/>'
+    rendered_attrs = ''.join([' %s="%s"' % (k, v) for k, v in kwargs.items()])
+    template = '<link rel="stylesheet" href="%s" type="text/css"%s>'
     return template % (href, rendered_attrs)
 
 
@@ -57,8 +58,9 @@ class PrecompilerAndAbsoluteFilterTestCase(SimpleTestCase):
 
     def setUp(self):
         self.html_orig = '<link rel="stylesheet" href="/static/css/relative_url.css" type="text/css" />'
-        self.html_link_to_precompiled_css = '<link rel="stylesheet" href="/static/CACHE/css/relative_url.e8602322bfa6.css" type="text/css" />'
-        self.html_link_to_absolutized_css = '<link rel="stylesheet" href="/static/CACHE/css/relative_url.376db5682982.css" type="text/css" />'
+        self.html_auto_close_removed = '<link rel="stylesheet" href="/static/css/relative_url.css" type="text/css">'
+        self.html_link_to_precompiled_css = '<link rel="stylesheet" href="/static/CACHE/css/relative_url.e8602322bfa6.css" type="text/css">'
+        self.html_link_to_absolutized_css = '<link rel="stylesheet" href="/static/CACHE/css/relative_url.376db5682982.css" type="text/css">'
         self.css_orig = "p { background: url('../img/python.png'); }" # content of relative_url.css
         self.css_absolutized = "p { background: url('/static/img/python.png?ccb38978f900'); }"
 
@@ -66,8 +68,10 @@ class PrecompilerAndAbsoluteFilterTestCase(SimpleTestCase):
         precompiler = (('text/css', 'compressor.tests.test_base.PassthroughPrecompiler'),) if use_precompiler else ()
         filters = ('compressor.filters.css_default.CssAbsoluteFilter',) if use_absolute_filter else ()
 
-        with self.settings(COMPRESS_ENABLED=enabled, COMPRESS_PRECOMPILERS=precompiler, COMPRESS_CSS_FILTERS=filters):
-            css_node = CssCompressor(self.html_orig)
+        with self.settings(COMPRESS_ENABLED=enabled,
+                           COMPRESS_PRECOMPILERS=precompiler,
+                           COMPRESS_FILTERS={'css': filters}):
+            css_node = CssCompressor('css', self.html_orig)
             output = list(css_node.hunks())[0]
             self.assertEqual(output, expected_output)
 
@@ -79,8 +83,8 @@ class PrecompilerAndAbsoluteFilterTestCase(SimpleTestCase):
         in the filters setting.
         While at it, ensure that everything runs as expected when compression is enabled.
         """
-        self.helper(enabled=False, use_precompiler=False, use_absolute_filter=False, expected_output=self.html_orig)
-        self.helper(enabled=False, use_precompiler=False, use_absolute_filter=True, expected_output=self.html_orig)
+        self.helper(enabled=False, use_precompiler=False, use_absolute_filter=False, expected_output=self.html_auto_close_removed)
+        self.helper(enabled=False, use_precompiler=False, use_absolute_filter=True, expected_output=self.html_auto_close_removed)
         self.helper(enabled=False, use_precompiler=True, use_absolute_filter=False, expected_output=self.html_link_to_precompiled_css)
         self.helper(enabled=False, use_precompiler=True, use_absolute_filter=True, expected_output=self.html_link_to_absolutized_css)
         self.helper(enabled=True, use_precompiler=False, use_absolute_filter=False, expected_output=self.css_orig)
@@ -98,22 +102,23 @@ class CompressorTestCase(SimpleTestCase):
 
     def setUp(self):
         self.css = """\
-<link rel="stylesheet" href="/static/css/one.css" type="text/css" />
+<link rel="stylesheet" href="/static/css/one.css" type="text/css">
 <style type="text/css">p { border:5px solid green;}</style>
-<link rel="stylesheet" href="/static/css/two.css" type="text/css" />"""
-        self.css_node = CssCompressor(self.css)
+<link rel="stylesheet" href="/static/css/two.css" type="text/css">"""
+        self.css_node = CssCompressor('css', self.css)
 
         self.js = """\
 <script src="/static/js/one.js" type="text/javascript"></script>
 <script type="text/javascript">obj.value = "value";</script>"""
-        self.js_node = JsCompressor(self.js)
+        self.js_node = JsCompressor('js', self.js)
 
     def assertEqualCollapsed(self, a, b):
         """
         assertEqual with internal newlines collapsed to single, and
         trailing whitespace removed.
         """
-        collapse = lambda x: re.sub(r'\n+', '\n', x).rstrip()
+        def collapse(s):
+            return re.sub(r'\n+', '\n', s).rstrip()
         self.assertEqual(collapse(a), collapse(b))
 
     def assertEqualSplits(self, a, b):
@@ -121,7 +126,8 @@ class CompressorTestCase(SimpleTestCase):
         assertEqual for splits, particularly ignoring the presence of
         a trailing newline on the content.
         """
-        mangle = lambda split: [(x[0], x[1], x[2], x[3].rstrip()) for x in split]
+        def mangle(split):
+            return [(x[0], x[1], x[2], x[3].rstrip()) for x in split]
         self.assertEqual(mangle(a), mangle(b))
 
     def test_css_split(self):
@@ -129,7 +135,7 @@ class CompressorTestCase(SimpleTestCase):
             (
                 SOURCE_FILE,
                 os.path.join(settings.COMPRESS_ROOT, 'css', 'one.css'),
-                'css/one.css', '<link rel="stylesheet" href="/static/css/one.css" type="text/css" />',
+                'css/one.css', '<link rel="stylesheet" href="/static/css/one.css" type="text/css">',
             ),
             (
                 SOURCE_HUNK,
@@ -141,7 +147,7 @@ class CompressorTestCase(SimpleTestCase):
                 SOURCE_FILE,
                 os.path.join(settings.COMPRESS_ROOT, 'css', 'two.css'),
                 'css/two.css',
-                '<link rel="stylesheet" href="/static/css/two.css" type="text/css" />',
+                '<link rel="stylesheet" href="/static/css/two.css" type="text/css">',
             ),
         ]
         split = self.css_node.split_contents()
@@ -161,7 +167,7 @@ class CompressorTestCase(SimpleTestCase):
         out = 'body { background:#990; }\n.compress-test {color: red;}'
         css = ("""<link rel="stylesheet" href="/static/css/one.css" type="text/css" />
         <link rel="stylesheet" href="/static/css/utf-8_with-BOM.css" type="text/css" />""")
-        css_node_with_bom = CssCompressor(css)
+        css_node_with_bom = CssCompressor('css', css)
         hunks = '\n'.join([h for h in css_node_with_bom.hunks()])
         self.assertEqual(out, hunks)
 
@@ -184,6 +190,11 @@ class CompressorTestCase(SimpleTestCase):
         output = css_tag('/static/CACHE/css/58a8c0714e59.css')
         self.assertEqual(output, self.css_node.output().strip())
 
+    def test_css_preload_output(self):
+        # this needs to have the same hash as in the test above
+        out = '<link rel="preload" href="/static/CACHE/css/58a8c0714e59.css" as="style" />'
+        self.assertEqual(out, self.css_node.output(mode="preload"))
+
     def test_js_split(self):
         out = [
             (
@@ -208,12 +219,17 @@ class CompressorTestCase(SimpleTestCase):
         self.assertEqual(out, list(self.js_node.hunks()))
 
     def test_js_output(self):
-        out = '<script type="text/javascript" src="/static/CACHE/js/74e158ccb432.js"></script>'
+        out = '<script src="/static/CACHE/js/8a0fed36c317.js"></script>'
         self.assertEqual(out, self.js_node.output())
 
+    def test_js_preload_output(self):
+        # this needs to have the same hash as in the test above
+        out = '<link rel="preload" href="/static/CACHE/js/8a0fed36c317.js" as="script" />'
+        self.assertEqual(out, self.js_node.output(mode="preload"))
+
     def test_js_override_url(self):
         self.js_node.context.update({'url': 'This is not a url, just a text'})
-        out = '<script type="text/javascript" src="/static/CACHE/js/74e158ccb432.js"></script>'
+        out = '<script src="/static/CACHE/js/8a0fed36c317.js"></script>'
         self.assertEqual(out, self.js_node.output())
 
     def test_css_override_url(self):
@@ -226,30 +242,30 @@ class CompressorTestCase(SimpleTestCase):
         self.assertEqualCollapsed(self.js, self.js_node.output())
 
     def test_js_return_if_on(self):
-        output = '<script type="text/javascript" src="/static/CACHE/js/74e158ccb432.js"></script>'
+        output = '<script src="/static/CACHE/js/8a0fed36c317.js"></script>'
         self.assertEqual(output, self.js_node.output())
 
     @override_settings(COMPRESS_OUTPUT_DIR='custom')
     def test_custom_output_dir1(self):
-        output = '<script type="text/javascript" src="/static/custom/js/74e158ccb432.js"></script>'
-        self.assertEqual(output, JsCompressor(self.js).output())
+        output = '<script src="/static/custom/js/8a0fed36c317.js"></script>'
+        self.assertEqual(output, JsCompressor('js', self.js).output())
 
     @override_settings(COMPRESS_OUTPUT_DIR='')
     def test_custom_output_dir2(self):
-        output = '<script type="text/javascript" src="/static/js/74e158ccb432.js"></script>'
-        self.assertEqual(output, JsCompressor(self.js).output())
+        output = '<script src="/static/js/8a0fed36c317.js"></script>'
+        self.assertEqual(output, JsCompressor('js', self.js).output())
 
     @override_settings(COMPRESS_OUTPUT_DIR='/custom/nested/')
     def test_custom_output_dir3(self):
-        output = '<script type="text/javascript" src="/static/custom/nested/js/74e158ccb432.js"></script>'
-        self.assertEqual(output, JsCompressor(self.js).output())
+        output = '<script src="/static/custom/nested/js/8a0fed36c317.js"></script>'
+        self.assertEqual(output, JsCompressor('js', self.js).output())
 
     @override_settings(COMPRESS_PRECOMPILERS=(
         ('text/foobar', 'compressor.tests.test_base.TestPrecompiler'),
     ), COMPRESS_ENABLED=True)
     def test_precompiler_class_used(self):
         css = '<style type="text/foobar">p { border:10px solid red;}</style>'
-        css_node = CssCompressor(css)
+        css_node = CssCompressor('css', css)
         output = make_soup(css_node.output('inline'))
         self.assertEqual(output.text, 'OUTPUT')
 
@@ -258,7 +274,7 @@ class CompressorTestCase(SimpleTestCase):
     ), COMPRESS_ENABLED=True)
     def test_nonexistent_precompiler_class_error(self):
         css = '<style type="text/foobar">p { border:10px solid red;}</style>'
-        css_node = CssCompressor(css)
+        css_node = CssCompressor('css', css)
         self.assertRaises(FilterDoesNotExist, css_node.output, 'inline')
 
     @override_settings(COMPRESS_PRECOMPILERS=(
@@ -266,7 +282,7 @@ class CompressorTestCase(SimpleTestCase):
     ), COMPRESS_ENABLED=True)
     def test_command_with_dot_precompiler(self):
         css = '<style type="text/foobar">p { border:10px solid red;}</style>'
-        css_node = CssCompressor(css)
+        css_node = CssCompressor('css', css)
         self.assertRaises(FilterError, css_node.output, 'inline')
 
     @override_settings(COMPRESS_PRECOMPILERS=(
@@ -274,7 +290,7 @@ class CompressorTestCase(SimpleTestCase):
     ), COMPRESS_ENABLED=True)
     def test_template_precompiler(self):
         css = '<style type="text/django">p { border:10px solid {% if 1 %}green{% else %}red{% endif %};}</style>'
-        css_node = CssCompressor(css)
+        css_node = CssCompressor('css', css)
         output = make_soup(css_node.output('inline'))
         self.assertEqual(output.text, 'p { border:10px solid green;}')
 
@@ -288,7 +304,7 @@ class CssMediaTestCase(SimpleTestCase):
 <style type="text/css">h1 { border:5px solid green;}</style>"""
 
     def test_css_output(self):
-        css_node = CssCompressor(self.css)
+        css_node = CssCompressor('css', self.css)
         links = make_soup(css_node.output()).find_all('link')
         media = ['screen', 'print', 'all', None]
         self.assertEqual(len(links), 4)
@@ -296,20 +312,20 @@ class CssMediaTestCase(SimpleTestCase):
 
     def test_avoid_reordering_css(self):
         css = self.css + '<style type="text/css" media="print">p { border:10px solid red;}</style>'
-        css_node = CssCompressor(css)
+        css_node = CssCompressor('css', css)
         media = ['screen', 'print', 'all', None, 'print']
         links = make_soup(css_node.output()).find_all('link')
         self.assertEqual(media, [l.get('media', None) for l in links])
 
     @override_settings(COMPRESS_PRECOMPILERS=(
-        ('text/foobar', 'python %s {infile} {outfile}' % os.path.join(test_dir, 'precompiler.py')),
+        ('text/foobar', '%s %s {infile} {outfile}' % (sys.executable, os.path.join(test_dir, 'precompiler.py'))),
     ), COMPRESS_ENABLED=False)
     def test_passthough_when_compress_disabled(self):
         css = """\
 <link rel="stylesheet" href="/static/css/one.css" type="text/css" media="screen">
 <link rel="stylesheet" href="/static/css/two.css" type="text/css" media="screen">
 <style type="text/foobar" media="screen">h1 { border:5px solid green;}</style>"""
-        css_node = CssCompressor(css)
+        css_node = CssCompressor('css', css)
         output = make_soup(css_node.output()).find_all(['link', 'style'])
         self.assertEqual(['/static/css/one.css', '/static/css/two.css', None],
                          [l.get('href', None) for l in output])
@@ -346,7 +362,7 @@ class JsAsyncDeferTestCase(SimpleTestCase):
                 return 'async'
             if tag.has_attr('defer'):
                 return 'defer'
-        js_node = JsCompressor(self.js)
+        js_node = JsCompressor('js', self.js)
         output = [None, 'async', 'defer', None, 'async', None]
         scripts = make_soup(js_node.output()).find_all('script')
         attrs = [extract_attr(s) for s in scripts]
@@ -361,11 +377,11 @@ class JSWithParensTestCase(SimpleTestCase):
         """
 
     def test_js_content(self):
-        js_node = JsCompressor(self.js)
+        js_node = JsCompressor('js', self.js)
 
         content = js_node.filter_input()
-        self.assertEqual(content[0], ';obj = {};')
-        self.assertEqual(content[1], ';pollos = {}')
+        self.assertEqual(content[0], 'obj = {};;')
+        self.assertEqual(content[1], 'pollos = {};')
 
 
 class CacheTestCase(SimpleTestCase):
@@ -430,8 +446,8 @@ class CompressorInDebugModeTestCase(SimpleTestCase):
             css.write(test_css_content)
         # We should generate a link with the hash of the original content, not
         # the modified one
-        expected = '<link rel="stylesheet" href="/static/CACHE/css/%s.css" type="text/css" />' % hashed
-        compressor = CssCompressor(self.css)
+        expected = '<link rel="stylesheet" href="/static/CACHE/css/%s.css" type="text/css">' % hashed
+        compressor = CssCompressor('css', self.css)
         compressor.storage = DefaultStorage()
         output = compressor.output()
         self.assertEqual(expected, output)
